/*

 ************************************
 ****[Internet Ralay Chat Library]***
 ************************************
 ** Easy Connect,                  **
 ** Easy Ident,                    **
 ** Easy Reteive and Send Messages **
 ************************************

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>

#define LI_VERSION "1.0b"

struct sockaddr_in sock_addr;
struct sockaddr_in6 sock_addr6;
struct hostent *host_ent;
enum stati { ERR_NICK=0, ERR_SERVER=1, ERR_USER=2, CONNECT=3, NO_ERR=4};
struct li_settings{
 	char nick[10];
 	char user[10];
 	char realname[51];
};

struct li_srvline{
	char nick[12];
	char user[12];
	char net[100];
	char dest[20];
	char msg[1024];
};

char *li_version(void);
int li_socket4(char *hostname, int port);
int li_connect4(char *hostname,int port,struct li_settings settings);
int li_socket6(char *hostname, int port);
int li_connect6(char *hostname,int port,struct li_settings settings);

void li_set_user(int sockfd,char *user, char *realname);
void li_set_nick(int sockfd, char *nick);
char *li_read(int sockfd);
void li_write(int sockfd,char *msg);
void li_pong(int sockfd,char *msg);
void li_getline(char *msg, struct li_srvline *srvline);
char li_randchar(void);
int li_ircerror(char *msg);
int li_join(int sockfd,char *channel,char *key);
int li_part(int sockfd,char *channel);
int li_say(int sockfd,char *dest,char *msg);

int close(int fd);
ssize_t write(int fd, const void *buf, size_t count);
ssize_t read(int fd, const void *buf, size_t count);
int inet_pton(int af, const char *src, void *dst);
int gettimeofday(struct timeval *tv, struct timezone *tz);

char *li_version(){
	return(LI_VERSION);
}
int li_socket4(char *hostname, int port){
	int sockfd;

	memset(&sock_addr,0,sizeof(struct sockaddr_in));

	if ((host_ent=gethostbyname(hostname)) == NULL){
			perror("gethostbyname function() ");
			exit(0);
	}

	memcpy((char *)&sock_addr.sin_addr,host_ent->h_addr,host_ent->h_length);

	sock_addr.sin_family = PF_INET;
	sock_addr.sin_port = htons(port);

	if((sockfd = socket(PF_INET, SOCK_STREAM,0))< 0){
		perror("socket function() ");
		close(sockfd);
		exit(0);
	}

	if(connect(sockfd,(struct sockaddr *)&sock_addr,sizeof(struct sockaddr_in))< 0){
		perror("connect function() ");
		close(sockfd);
		exit(0);
	}
	return(sockfd);
}

int li_connect4(char *hostname,int port,struct li_settings settings){
	int sockfd;

	sockfd=li_socket4(hostname,port);
	li_set_user(sockfd,settings.user, settings.realname);
	li_set_nick(sockfd,settings.nick);
	while(strstr(li_read(sockfd),"433 *")){
		char nick[5];
		int i;
		for(i=0; i<=5;i++) nick[i]= li_randchar();
		li_set_nick(sockfd,nick);
	}
	return(sockfd);
}


int li_socket6(char *hostname, int port){
	int sockfd;

	memset(&sock_addr6,0,sizeof(struct sockaddr_in6));

	inet_pton(AF_INET6,hostname,(struct sockaddr_in6 *)&sock_addr6.sin6_addr) ;

	sock_addr6.sin6_family = PF_INET6;
	sock_addr6.sin6_port = htons(port);

	if((sockfd = socket(PF_INET6, SOCK_STREAM,0))< 0){
		printf("socket function() ");
		close(sockfd);
		exit(0);
	}
	if((connect(sockfd,(struct sockaddr *)&sock_addr6,sizeof(sock_addr6)))<0){
		printf("connect function() ");
		close(sockfd);
		exit(0);
	}
	return(sockfd);
}

int li_connect6(char *hostname,int port,struct li_settings settings){
	int sockfd;

	sockfd=li_socket6(hostname,port);
	li_set_user(sockfd,settings.user, settings.realname);
	li_set_nick(sockfd,settings.nick);
	while(strstr(li_read(sockfd),"433 *")){
		char nick[5];
		int i;
		for(i=0; i<=5;i++) nick[i]= li_randchar();
		li_set_nick(sockfd,nick);
	}
	return(sockfd);
}

void li_set_user(int sockfd,char *user, char *realname){
	char cmd[1024];
	memset(&cmd,0,1024);
	sprintf(cmd,"USER %s 8 * : %s\n",user, realname);
	write(sockfd,cmd,strlen(cmd));
	return;
}

void li_set_nick(int sockfd, char *nick) {
	char cmd[1024];
	memset(&cmd,0,1024);
	sprintf(cmd,"NICK %s\n",nick);
	write(sockfd,cmd,strlen(cmd));
	return;
}

char *li_read(int sockfd){
	char msg[1024];
	char *p_msg;
	memset(&msg,0,1024);
	read(sockfd,msg,1024);
	p_msg=msg;
	return(p_msg);
}

void li_write(int sockfd,char *msg){
	char cmd[1024];
	memset(&cmd,0,1024);
	memcpy(&cmd,msg,strlen(msg));
	write(sockfd,cmd,strlen(cmd));
	return;
}

void li_pong(int sockfd,char *msg){
	char ping[50], server[20];
	memset(&ping,0,50);
	memset(&server,0,20);
	sscanf(msg,"PING :%s\n",(char *)&server);
	sprintf(ping,"PONG %s\n",server);
	write(sockfd,ping,strlen(ping));
	return;
}

void li_getline(char *msg, struct li_srvline *srvline){
	msg++;
	strncpy(srvline->nick,strtok(msg,"!"),12);
	while(*(msg++)!='\0');
	strncpy(srvline->user,strtok(msg,"@"),12);
	while(*(msg++)!='\0');
	strncpy(srvline->net,strtok(msg," "),100);
	while(*(msg++)!='\0');
	msg++;
	while(*(msg++)!=' ');
	strncpy(srvline->dest,strtok(msg," "),20);
	while(*(msg++)!='\0');
	msg++;
	strncpy(srvline->msg,strtok(msg,"\r"),1024);
}

void li_tolower(char *msg){

}

char li_randchar(void){
	char rc;
	struct timeval tv;
	unsigned char charset[] = "abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ123456789\0";
	gettimeofday(&tv, 0);
	srand(tv.tv_usec);
	rc=charset[(int)(rand() % strlen(charset))];
	return(rc);
}

int li_ircerror(char *msg){
return(1);
}

int li_join(int sockfd,char *channel,char *key){
	char join_cmd[500];
	memset(join_cmd,0,500);
	if (strstr(channel,"#"))
		sprintf(join_cmd,"join %s %s\n",channel,key);
	else
		sprintf(join_cmd,"join #%s %s\n",channel,key);
	li_write(sockfd,join_cmd);
	return(1);
}
int li_part(int sockfd,char *channel){
	char part_cmd[500];
	memset(part_cmd,0,500);
	if (strstr(channel,"#"))
		sprintf(part_cmd,"part %s\n",channel);
	else
		sprintf(part_cmd,"part #%s\n",channel);
	li_write(sockfd,part_cmd);
	return(1);
}

int li_say(int sockfd,char *dest,char *msg){
	char say_cmd[500];
	memset(say_cmd,0,500);
	sprintf(say_cmd,"privmsg %s :%s\n",dest,msg);
	li_write(sockfd,say_cmd);
	return(1);
}


